@@ -53,10 +53,7 @@ class BtDependencyTest:public CppUnit::TestFixture {
       (new DownloadContext(1024*1024, length, torrentFile));
     dctx->setDir(".");
     dependee->setDownloadContext(dctx);
-    DefaultPieceStorageHandle ps(new DefaultPieceStorage(dctx, option.get()));
-    dependee->setPieceStorage(ps);
-    ps->initStorage();
-    dependee->initSegmentMan();
+    dependee->initPieceStorage();
     return dependee;
   }
 
@@ -94,7 +91,7 @@ void BtDependencyTest::testResolve()
   const SharedHandle<FileEntry>& firstFileEntry =
     dependant->getDownloadContext()->getFirstFileEntry();
   CPPUNIT_ASSERT_EQUAL(std::string("/tmp/outfile.path"),
-		       firstFileEntry->getPath());
+                       firstFileEntry->getPath());
   CPPUNIT_ASSERT_EQUAL((size_t)1, firstFileEntry->getRemainingUris().size());
 }
 
@@ -139,7 +136,7 @@ void BtDependencyTest::testResolve_loadError()
   CPPUNIT_ASSERT
     (!dependant->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT));
   CPPUNIT_ASSERT_EQUAL(std::string("/tmp/outfile.path"),
-		       dependant->getFirstFilePath());
+                       dependant->getFirstFilePath());
 }
 
 void BtDependencyTest::testResolve_dependeeFailure()
@@ -153,7 +150,7 @@ void BtDependencyTest::testResolve_dependeeFailure()
   CPPUNIT_ASSERT
     (!dependant->getDownloadContext()->hasAttribute(bittorrent::BITTORRENT));
   CPPUNIT_ASSERT_EQUAL(std::string("/tmp/outfile.path"),
-		       dependant->getFirstFilePath());
+                       dependant->getFirstFilePath());
 }
 
 void BtDependencyTest::testResolve_dependeeInProgress()
@@ -167,7 +164,7 @@ void BtDependencyTest::testResolve_dependeeInProgress()
   BtDependency dep(dependant, dependee);
   CPPUNIT_ASSERT(!dep.resolve());
   CPPUNIT_ASSERT_EQUAL(std::string("/tmp/outfile.path"),
-		       dependant->getFirstFilePath());
+                       dependant->getFirstFilePath());
 }
 
 } // namespace aria2
